#!/bin/bash
# Copyright (C) 2024 Atomicorp, Inc.
# All rights reserved.
#
# Version: 1.0

# Check for root privileges
if [ "$(id -u)" != "0" ]; then
    echo "$(date) ERROR: This script must be run as root" >> ${LOG_FILE}
    exit 1
fi

LOCAL=`dirname $0`
cd $LOCAL
cd ../
PWD=`pwd`

# Log location
LOG_FILE="/var/ossec/logs/active-responses.log"

# Isolation hosts file
ISOLATION_HOSTS="/var/ossec/etc/shared/isolation-hosts"

# Enable debug logging
DEBUG=0

log() {
    echo "$(date) $1" >> ${LOG_FILE}
}

debug_log() {
    if [ "$DEBUG" = "1" ]; then
        log "DEBUG: $1"
    fi
}

# Function to run iptables commands with error checking
run_iptables_cmd() {
    local cmd="$1"
    debug_log "Running iptables command: $cmd"
    output=$($cmd 2>&1)
    ret=$?
    if [ $ret -ne 0 ]; then
        log "ERROR: iptables command failed: $cmd"
        log "ERROR: Output: $output"
        return 1
    fi
    if [ -n "$output" ]; then
        debug_log "Command output: $output"
    fi
    return 0
}

# Read input and determine action
if [ "$1" == "add" ] || [ "$1" == "delete" ]; then
    ACTION=$1
    USER=$2
    IP=$3
    debug_log "Running with command line args: ACTION=$ACTION USER=$USER IP=$IP"
    
    if [[ "$IP" == "isolate:disable" ]]; then
        ACTION="delete"
    fi
else
    read -r stdin_data
    debug_log "Received stdin data: $stdin_data"
    if [[ "$stdin_data" == *"enable"* ]]; then
        ACTION="add"
    elif [[ "$stdin_data" == *"disable"* ]]; then
        ACTION="delete"
    else
        log "ERROR: Invalid input - could not determine action"
        exit 1
    fi
    debug_log "Determined ACTION=$ACTION from stdin"
fi

# Check if isolation-hosts file exists
if [ ! -f "$ISOLATION_HOSTS" ]; then
    log "ERROR: Isolation hosts file not found at $ISOLATION_HOSTS"
    exit 1
fi
debug_log "Found isolation hosts file at $ISOLATION_HOSTS"

# Function to apply isolation rules
apply_isolation() {
    debug_log "Starting apply_isolation function"
    
    # Read isolation hosts into array using compatible method
    i=0
    while IFS= read -r line; do
        if [ -n "$line" ]; then
            ALLOWED_IPS[$i]="$line"
            debug_log "Added IP to allow list: ${ALLOWED_IPS[$i]}"
            i=$((i + 1))
        fi
    done < "$ISOLATION_HOSTS"
    
    # Always include localhost IP
    ALLOWED_IPS[$i]="127.0.0.1"
    debug_log "Total IPs in allow list: $((i + 1))"

    # Check for iptables vs ipchains
    debug_log "Checking for available firewall commands"
    if command -v iptables >/dev/null 2>&1; then
        debug_log "Found iptables command"
        debug_log "Current iptables rules before changes:"
        iptables-save >> ${LOG_FILE} 2>&1
        
        # Flush existing rules
        debug_log "Flushing existing iptables rules"
        run_iptables_cmd "iptables -F" || exit 1
        
        debug_log "Setting default policies to DROP"
        run_iptables_cmd "iptables -P INPUT DROP" || exit 1
        run_iptables_cmd "iptables -P OUTPUT DROP" || exit 1
        run_iptables_cmd "iptables -P FORWARD DROP" || exit 1

        # Allow loopback
        debug_log "Adding loopback rules"
        run_iptables_cmd "iptables -A INPUT -i lo -j ACCEPT" || exit 1
        run_iptables_cmd "iptables -A OUTPUT -o lo -j ACCEPT" || exit 1

        # Allow established connections
        debug_log "Adding established connection rules"
        run_iptables_cmd "iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT" || exit 1
        run_iptables_cmd "iptables -A OUTPUT -m state --state ESTABLISHED,RELATED -j ACCEPT" || exit 1

        # Allow traffic to/from isolation hosts
        debug_log "Adding rules for allowed IPs"
        for ip in "${ALLOWED_IPS[@]}"; do
            # Skip empty lines and comments
            if [ -z "$ip" ] || [ "${ip:0:1}" = "#" ]; then
                debug_log "Skipping empty/comment line: $ip"
                continue
            fi
            
            debug_log "Adding rules for IP: $ip"
            run_iptables_cmd "iptables -A INPUT -s $ip -j ACCEPT" || exit 1
            run_iptables_cmd "iptables -A OUTPUT -d $ip -j ACCEPT" || exit 1
        done
        
        debug_log "Listing final iptables rules:"
        iptables-save >> ${LOG_FILE} 2>&1
        log "Applied Linux isolation rules (iptables) successfully"
    elif command -v ipchains >/dev/null 2>&1; then
        debug_log "Found ipchains command"
        # Legacy ipchains support
        debug_log "Flushing existing ipchains rules"
        ipchains -F
        debug_log "Setting default policies to DENY"
        ipchains -P input DENY
        ipchains -P output DENY
        ipchains -P forward DENY

        # Allow loopback
        debug_log "Adding loopback rules"
        ipchains -A input -i lo -j ACCEPT
        ipchains -A output -i lo -j ACCEPT

        # Allow established connections
        debug_log "Adding established connection rules"
        ipchains -A input -j ACCEPT -y

        # Allow traffic to/from isolation hosts
        debug_log "Adding rules for allowed IPs"
        for ip in "${ALLOWED_IPS[@]}"; do
            if [ -z "$ip" ] || [ "${ip:0:1}" = "#" ]; then
                debug_log "Skipping empty/comment line: $ip"
                continue
            fi
            debug_log "Adding rules for IP: $ip"
            ipchains -A input -s $ip -j ACCEPT
            ipchains -A output -d $ip -j ACCEPT
        done
        debug_log "Listing final ipchains rules:"
        ipchains -L >> ${LOG_FILE} 2>&1
        log "Applied Linux isolation rules (ipchains) successfully"
    else
        log "ERROR: No supported firewall found (requires iptables or ipchains)"
        debug_log "Neither iptables nor ipchains commands were found"
        exit 1
    fi
}

# Function to remove isolation rules
remove_isolation() {
    debug_log "Starting remove_isolation function"
    if command -v iptables >/dev/null 2>&1; then
        debug_log "Found iptables command"
        debug_log "Flushing all rules"
        iptables -F
        debug_log "Setting default policies to ACCEPT"
        iptables -P INPUT ACCEPT
        iptables -P OUTPUT ACCEPT
        iptables -P FORWARD ACCEPT
        debug_log "Listing final iptables rules:"
        iptables -L -n >> ${LOG_FILE} 2>&1
        log "Removed Linux isolation rules (iptables) successfully"
    elif command -v ipchains >/dev/null 2>&1; then
        debug_log "Found ipchains command"
        debug_log "Flushing all rules"
        ipchains -F
        debug_log "Setting default policies to ACCEPT"
        ipchains -P input ACCEPT
        ipchains -P output ACCEPT
        ipchains -P forward ACCEPT
        debug_log "Listing final ipchains rules:"
        ipchains -L >> ${LOG_FILE} 2>&1
        log "Removed Linux isolation rules (ipchains) successfully"
    else
        log "ERROR: No supported firewall found (requires iptables or ipchains)"
        debug_log "Neither iptables nor ipchains commands were found"
        exit 1
    fi
}

# Main execution
debug_log "Starting main execution with ACTION=$ACTION"
case $ACTION in
    "add")
        log "Applying isolation rules"
        apply_isolation
        ;;
    "delete")
        log "Removing isolation rules"
        remove_isolation
        ;;
    *)
        log "Invalid action: $ACTION"
        exit 1
        ;;
esac

debug_log "Script completed successfully"
exit 0 