#!/bin/bash
# Copyright Atomicorp, 2025
# All Rights Reserved

###############
# Globals
###############
DEBUG=0
LOG_FILE="/var/ossec/logs/atomicorp-api.log"
MODULE_NAME="atomicorp-api-module-clamav"
CUSTOM_SCAN_CONF=0  # Initialize the variable

source /var/ossec/lib/atomicorp-functions.sh


#######################
# Main
#######################
if [[ $DEBUG -ge 1 ]]; then
	log_event "DEBUG1: $0: START"
fi

if [ ! -f /var/ossec/etc/client.keys ]; then
    echo "Error: Agent not registered"
	log_event "ERROR: Agent not registered"
    exit 1
fi
ID=$(cat /var/ossec/etc/client.keys | awk '{print $1}')
if [ -f /var/ossec/etc/shared/malware-detection ]; then
    CONFIG_FILE="/var/ossec/etc/shared/malware-detection"
elif [ -f /var/ossec/etc/shared/${ID}.malware-detection ]; then
    CONFIG_FILE="/var/ossec/etc/shared/${ID}.malware-detection"
else
	if [[ $DEBUG -ge 1 ]]; then
		log_event "ERROR: Neither ${ID}.malware-detection nor malware-detection found."
	fi
    exit 
fi

if [[ $DEBUG -ge 1 ]]; then
	log_event "DEBUG1: $0: config file is ($CONFIG_FILE) and clamd.conf is ($CLAM_CONF)"
fi


if [ -f "$CONFIG_FILE" ]; then
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: Updating clamav settings: START"
	fi

	# Detect distribution, and clamd 
	dist_detection
	clamd_version

	if  [ -f /var/ossec/etc/shared/clamd-scan.conf.template ]; then
        if grep DISABLED /var/ossec/etc/shared/clamd-scan.conf.template >/dev/null 2>&1; then
            echo
            echo
            echo "############################################################################################"
            echo "CLAMAV MODULE DISABLED"
            echo "############################################################################################"
            echo
            echo
            exit 0
        fi
    fi

	# Is this enabled
	if grep -q ^setting,enable,true $CONFIG_FILE; then
		ENABLE=1
		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG1: clamav ENABLED is $ENABLE"
		fi

		# Copy in templates
		clamd_setup_and_settings
		if [[ $? -ne 0 ]]; then
			if [[ $DEBUG -ge 1 ]]; then
				log_event "ERROR: clamd_setup_and_settings failed or disabled"
			fi
			exit 0
		fi

		# If this uses a standard template, it will be managed. Custom templates are not managed
		if [ $CUSTOM_SCAN_CONF -eq 0 ]; then
		    $SED -i "s/^User.*/User root/g" $CLAM_CONF
		    $SED -i "s/^LocalSocketGroup.*//g" $CLAM_CONF
		    $SED -i "s/^LocalSocketMode.*//g" $CLAM_CONF
		    $SED -i "s/^OnAccessExcludeRootUID.*//g" $CLAM_CONF
		    $SED -i "s/^OnAccessExcludeUID.*//g" $CLAM_CONF
		    $SED -i "s/^OnAccessIncludePath.*//g" $CLAM_CONF
		    $SED -i "s/^OnAccessExcludePath.*//g" $CLAM_CONF
		    $SED -i "s/^OnAccessPrevention.*//g" $CLAM_CONF


		    for line in $(cat $CONFIG_FILE ); do

		    	if [[ $DEBUG -ge 2 ]]; then
		    		log_event "DEBUG2: input is: ($line)"
		    	fi

		    	if  [[ ${line} == uid* ]]; then
		    		NAME=$(echo $line |$AWK -F, '{ print $2}')
		    		if id -u ${NAME} >/dev/null 2>&1 ; then
		    			CLAM_UID=$(id -u ${NAME})
		    			if [[ ${CLAM_UID} -eq 0 ]]; then
		    				echo "OnAccessExcludeRootUID yes"  >> $CLAM_CONF
		    			else
		    				echo "OnAccessExcludeUID ${CLAM_UID}" >> $CLAM_CONF
		    			fi
		    		fi
		    	elif [[ ${line} == include* ]]; then
		    		CLAM_INCLUDE=$(echo $line |$AWK -F, '{ print $2}')
		    		if [ -d $CLAM_INCLUDE ]; then
		    				echo "OnAccessIncludePath  $CLAM_INCLUDE" >> $CLAM_CONF
		    		fi
		    	    INCLUDE_PATHS=$((INCLUDE_PATHS + 1))
		    	elif [[ ${line} == exclude* ]]; then
		    		CLAM_EXCLUDE=$(echo $line |$AWK -F, '{ print $2}')
		    		echo "OnAccessExcludePath $CLAM_EXCLUDE" >> $CLAM_CONF
		    	elif [[ ${line} == setting,* ]]; then

		    		CLAM_SETTING=$(echo $line |$AWK -F, '{ print $2}')
		    		CLAM_TOKEN=$(echo $line |$AWK -F, '{ print $3}')

		    		# Freshclam settings
		    		if [[ ${CLAM_SETTING} == freshclam_* ]]; then
		    			# if CLAM_SETTING is freshclam_feed_type, and CLAM_TOKEN is atomic then we will process values for linux_sigs, windows_sigs, and android_sigs
		    			if [[ ${CLAM_SETTING} == "freshclam_feed_type" ]] && [[ ${CLAM_TOKEN} == "atomic" ]]; then
		    				FRESHCLAM_FEED_TYPE="atomic"
		    			elif [[ ${CLAM_SETTING} == "freshclam_feed_type" ]] && [[ ${CLAM_TOKEN} == "official" ]]; then
		    				FRESHCLAM_FEED_TYPE="official"
		    			fi

		    			if [[ ${CLAM_SETTING} == "freshclam_linux_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
		    				FRESHCLAM_LINUX_SIGS="yes"
		    			fi
		    			if [[ ${CLAM_SETTING} == "freshclam_windows_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
		    				FRESHCLAM_WINDOWS_SIGS="yes"
		    			fi
		    			if [[ ${CLAM_SETTING} == "freshclam_android_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
		    				FRESHCLAM_ANDROID_SIGS="yes"
		    			fi
		    			if [[ ${CLAM_SETTING} == "freshclam_osx_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
		    				FRESHCLAM_OSX_SIGS="yes"
		    			fi
		    		elif [[ ${CLAM_SETTING} == "quarantine" ]]; then
		    			if [[ $CLAM_TOKEN == "true" ]]; then
		    				QUARANTINE=1
		    			fi
		    		elif [[ ${CLAM_SETTING} == "quarantine_path" ]]; then
		    			QUARANTINE_PATH=${CLAM_TOKEN}
		    		elif [[ ${CLAM_SETTING} == "enable" ]]; then
		    			if [[ $DEBUG -ge 1 ]]; then
		    				log_event "DEBUG1: CLAM_SETTING is ${CLAM_TOKEN}"
		    			fi
		    		else 
		    		    if grep -q "^${CLAM_SETTING}" $CLAM_CONF; then
		    			$SED -i "s/^${CLAM_SETTING}.*/${CLAM_SETTING} ${CLAM_TOKEN}/g" $CLAM_CONF
		    		    else
		    			echo "${CLAM_SETTING} ${CLAM_TOKEN}" >> $CLAM_CONF
		    		    fi

		    		    if [[ ${CLAM_SETTING} == "OnAccessPrevention" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
		    			REALTIME=1
		    		    fi

		    		fi
		    				
		    	fi

		    done

		    # Generic fixes
		    $SED -i '/^$/d' $CLAM_CONF
		    # Solaris fixes
		    if [[ $DIST == "solaris" ]]; then
			$SED -i '/^StructuredCCOnly.*/d' $CLAM_CONF
		    fi

		
		    freshclam_settings
		    freshclam_update
		    quarantine_settings
		fi
    else
	    if [[ $DEBUG -ge 1 ]]; then
	    	log_event "DEBUG1: clamav ENABLED is $ENABLE"
	    fi
    	ENABLE=0
    fi

	# Restart the service function
	clamd_restart

	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: clamav settings updated: END"
	fi

else
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: CLAM config $CONFIG_FILE and $CLAM_CONF not detected"
	fi

	
fi

if [[ $DEBUG -ge 1 ]]; then
	log_event "DEBUG1: $0: END"
fi

