#!/bin/bash
# Copyright Atomicorp, 2025
# All Rights Reserved



# Globals
VERSION="0.3"
FW_POLICY=/var/ossec/etc/shared/firewall-policy
DEBUG=${DEBUG:-0}
LOG_FILE="/var/ossec/logs/atomicorp-api.log"
MODULE_NAME="atomicorp-api-module-firewall"

# Source common functions
. /var/ossec/lib/atomicorp-functions.sh

# Get agent ID
if [ -f /var/ossec/etc/client.keys ]; then
    ID=$(cat /var/ossec/etc/client.keys | awk '{print $1}')
else
    log_event "ERROR: Agent not registered"
    exit 1
fi

# Validate policy file format
validate_policy() {
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Validating policy file format"
	if [ ! -f "$FW_POLICY" ]; then
		log_event "ERROR: Firewall policy file not found at $FW_POLICY"
		exit 1
	fi
	
	# Check if file is readable
	if [ ! -r "$FW_POLICY" ]; then
		log_event "ERROR: Firewall policy file is not readable"
		exit 1
	fi
	
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Policy file validation passed"
}

#AIX
function aix_firewall() {
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Starting AIX firewall configuration"

	# Dependencies
	#   bos.net.ipsec.keymgt
	#   bos.net.ipsec.rte
	if ! lslpp -l |egrep -q "bos.net.ipsec.keymgt|bos.net.ipsec.rte" ; then
		log_event "ERROR: A2011: Firewall (AIX) support is not available"
		exit 1
	fi
	RMFILT="/usr/sbin/rmfilt"
	GENFILT="/usr/sbin/genfilt"
	MKFILT="/usr/sbin/mkfilt"
	LSFILT="/usr/sbin/lsfilt"

	## Clear existing policy
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Clearing existing AIX firewall rules"
	$RMFILT -v 4 -n all
	$RMFILT -v 6 -n all

	# Initialize firewall
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Initializing AIX firewall"
	$MKFILT -c ipsec -t 4 >/dev/null 2>&1
	$MKFILT -v 4 -u -z P
	$MKFILT -v 6 -u -z P

	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Processing AIX firewall rules"
	while read line ; do
		# Skip comments and empty lines
		[[ "$line" =~ ^#.*$ || -z "$line" ]] && continue

		IPV=$(echo $line | awk -F, '{print $2}')
		SRCIP=$(echo $line | awk -F, '{print $3}')
		SRCIP_MASK=$(echo $line | awk -F, '{print $4}')
		DSTIP=$(echo $line | awk -F, '{print $5}')
		DSTIP_MASK=$(echo $line | awk -F, '{print $6}')
		PROTOCOL=$(echo $line | awk -F, '{print $7}')
		CONDITION=$(echo $line | awk -F, '{print $8}')
		PORT=$(echo $line | awk -F, '{print $9}')
		DIRECTION=$(echo $line | awk -F, '{print $10}')
		
		[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Processing rule: proto=$PROTOCOL port=$PORT dir=$DIRECTION src=$SRCIP dst=$DSTIP"
		
		if [[ $DIRECTION == inbound ]]; then
			DIRECTION=I 
		elif [[ $DIRECTION == outbound ]]; then
			DIRECTION=O
		fi
		
		COMMENT=$(echo $line | awk -F, '{print $11}')
		LOG=$(echo $line | awk -F, '{print $12}')
		if [[ $LOG == log ]]; then
			LOG=Y
		else
			LOG=N
		fi

		if  [[ ${line} == disabled* ]]; then
			[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Skipping disabled rule"
			continue
		elif  [[ ${line} == permit* ]]; then
			[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Adding permit rule"
			$GENFILT -v $IPV -a P -s $SRCIP -m $SRCIP_MASK -d $DSTIP -M $DSTIP_MASK -c $PROTOCOL -O $CONDITION -P $PORT -w $DIRECTION -D "${COMMENT}" -l $LOG
		elif [[ ${line} == deny* ]]; then
			[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Adding deny rule"
			$GENFILT -v $IPV -a D -s $SRCIP -m $SRCIP_MASK -d $DSTIP -M $DSTIP_MASK -c $PROTOCOL -O $CONDITION -P $PORT -w $DIRECTION -D "${COMMENT}" -l $LOG
		fi
	done < $FW_POLICY

	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Activating AIX firewall rules"
	$MKFILT -g start
	$MKFILT -v4 -u
	$MKFILT -v6 -u
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: AIX firewall configuration completed"
}

function rhel_firewall() {
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Starting RHEL firewall configuration"
	
	# Check if firewalld is running and enabled
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Checking firewalld status"
	if ! systemctl is-active --quiet firewalld; then
		[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Firewalld not running, starting service"
		systemctl start firewalld
		systemctl enable firewalld
	fi

	# Clear ALL existing configuration
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Removing all existing firewall configuration"
	
	# Remove all rules from public zone
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Clearing public zone configuration"
	
	# Get current rich rules and remove them one by one
	for rule in $(firewall-cmd --permanent --zone=public --list-rich-rules); do
		firewall-cmd --permanent --zone=public --remove-rich-rule="$rule"
	done

	# Get current services and remove them one by one
	for service in $(firewall-cmd --permanent --zone=public --list-services); do
		firewall-cmd --permanent --zone=public --remove-service="$service"
	done

	# Get current ports and remove them one by one
	for port in $(firewall-cmd --permanent --zone=public --list-ports); do
		firewall-cmd --permanent --zone=public --remove-port="$port"
	done

	# Remove other configurations
	firewall-cmd --permanent --zone=public --remove-forward-port=all >/dev/null 2>&1
	firewall-cmd --permanent --zone=public --remove-masquerade >/dev/null 2>&1
	firewall-cmd --permanent --zone=public --remove-protocol=all >/dev/null 2>&1
	firewall-cmd --permanent --zone=public --remove-source-port=all >/dev/null 2>&1
	firewall-cmd --permanent --zone=public --remove-icmp-block=all >/dev/null 2>&1

	# First pass - handle default policies
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Processing default policies"
	while IFS=, read -r type rest; do
		# Skip comments and empty lines
		[[ "$type" =~ ^#.*$ || -z "$type" ]] && continue
		
		case "$type" in
			default)
				chain=$(echo "$rest" | cut -d, -f1)
				policy=$(echo "$rest" | cut -d, -f2)
				[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Setting default policy for chain $chain to $policy"
				case "$chain" in
					input)  
						# For input chain, set the default zone
						[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Setting default zone to $policy"
						firewall-cmd --permanent --set-default-zone="$policy"
						;;
				esac
				;;
		esac
	done < "$FW_POLICY"
	
	# Second pass - handle rules
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Processing firewall rules"
	while IFS=, read -r type rest; do
		# Skip comments and empty lines
		[[ "$type" =~ ^#.*$ || -z "$type" ]] && continue
		
		case "$type" in
			rule)
				action=$(echo "$rest" | cut -d, -f1)
				protocol=$(echo "$rest" | cut -d, -f2)
				port=$(echo "$rest" | cut -d, -f3)
				direction=$(echo "$rest" | cut -d, -f4)
				source=$(echo "$rest" | cut -d, -f5)
				dest=$(echo "$rest" | cut -d, -f6)
				log=$(echo "$rest" | cut -d, -f7)
				
				[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Processing rule: $action $protocol port $port dir=$direction src=$source dst=$dest log=$log"
				
				# Skip non-input rules for now
				if [ "$direction" != "in" ]; then
					continue
				fi

				# Convert action to lowercase for rich rules
				action=${action,,}  # Convert to lowercase
				[ "$action" = "allow" ] && action="accept"
				
				# Handle port ranges
				if [[ $port == *-* ]]; then
					start_port=${port%-*}
					end_port=${port#*-}
					port="$start_port-$end_port"
				fi

				# If source is "any", add a simple port rule
				if [ "$source" = "any" ]; then
					[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Adding port rule for $protocol/$port"
					firewall-cmd --permanent --zone=public --add-port="${port}/${protocol}"
				else
					# For specific sources, add rich rules
					for src in $(echo "$source" | tr ';' ' '); do
						rich_rule="rule family=\"ipv4\" source address=\"${src}\""
						
						# Add destination if specified and not "any"
						if [ "$dest" != "any" ]; then
							rich_rule="$rich_rule destination address=\"$dest\""
						fi
						
						# Add port and protocol
						rich_rule="$rich_rule port port=\"$port\" protocol=\"$protocol\""
						
						# Add logging if enabled
						if [ "$log" = "yes" ]; then
							rich_rule="$rich_rule log prefix=\"${action}-${port}-${protocol}\""
						fi
						
						# Add action (must be lowercase)
						rich_rule="$rich_rule $action"
						
						[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Adding rich rule: $rich_rule"
						firewall-cmd --permanent --zone=public --add-rich-rule="$rich_rule"
					done
				fi
				;;
		esac
	done < "$FW_POLICY"
	
	# Reload firewall to apply all changes
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Reloading firewall to apply changes"
	firewall-cmd --reload

	# Verify final state
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Final firewall state:"
	[[ $DEBUG -ge 1 ]] && nft list ruleset >> /var/ossec/logs/atomicorp-api.log
	
	[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Firewall configuration completed"
}

# Main
[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Starting firewall module version $VERSION"

validate_policy

[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Detecting distribution"
dist_detection
[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Detected distribution: $DIST"

# Determine firewall implementation based on distribution
case "$DIST" in
    "el7"|"el8"|"el9"|"amazon")
        [[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Using RHEL firewall configuration"
        rhel_firewall
        ;;
    "aix")
        [[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Using AIX firewall configuration"
        aix_firewall
        ;;
    *)
        log_event "ERROR: A1001: Unsupported distribution: $DIST"
        log_event "ERROR: Supported distributions: RHEL/CentOS/Rocky 7/8/9, Amazon Linux, AIX"
        exit 1
        ;;
esac

[[ $DEBUG -ge 1 ]] && log_event "DEBUG1: Firewall module completed successfully"

