#!/usr/bin/bash
# Atomicorp Malware Scanner Active Response Script
#
# Copyright Atomicorp 2024
# All Rights reserved


#############################
# Globals
#############################
LOG_FILE="/var/ossec/logs/active-responses.log"
DEBUG=0


source /var/ossec/lib/atomicorp-functions.sh

if [ -f /etc/default/clamav-daemon ]; then
	source /etc/default/clamav-daemon
fi

# Planned
if [ -f /var/ossec/etc/malware-scanner.conf ]; then
	source /var/ossec/etc/malware-scanner.conf
else
	CLAMAV_SERVER=daemon
fi


# Detect Fedora/ RedHat 9 style clamav 
if [ -f /etc/freshclam.conf ] && [ -f /etc/clamd.d/scan.conf ]; then
    DISTRO=rh9like
fi

###########################
#functions
###########################


function init_clamav_conf() {
	if [[ $DIST == "aix"* ]]; then
		cp -f /var/ossec/modules/clamav/template/clamd.conf.template /opt/freeware/etc/clamd.conf
		cp -f /var/ossec/modules/clamav/template/freshclam.conf.template /opt/freeware/etc/freshclam.conf
		if [ ! -d /var/log ]; then
			mkdir -p /var/log
		fi

	else
		/var/ossec/modules/clamav/clam-module.sh
	fi
}


function install_clamav(){
    if [[ $DIST == "aix"* ]]; then
		echo
	elif [[ $DISTRO == "rh9like" ]]; then
		if ! rpm -q clamd ; then
			yum install -y clamd
		fi
		if ! rpm -q clamav ; then
			yum install -y clamav
		fi
	else
		if ! rpm -q clamd ; then
			yum -y install clamd
		fi
	fi
}

# add a function to configure remote clamav server  
function init_clamav_remote_conf(){
	if [[ $DIST == "aix"* ]]; then
		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG0: init_clamav_remote_conf AIX"
		fi

		# Configure stunnel.conf
		cp -f /var/ossec/modules/clamav/template/stunnel.conf /opt/freeware/etc/stunnel/stunnel.conf
		/opt/freeware/bin/sed -i "s/CLAMAV_SERVER/${HUB_IP}/g" /opt/freeware/etc/stunnel/stunnel.conf

		# Configure clamd.conf
		cp -f /var/ossec/modules/clamav/template/clamd.remote.conf /opt/freeware/etc/clamd.conf

		if [ ! -d /var/log/clamav ]; then
			mkdir -p /var/log/clamav
		fi

		# add stunnel-aix.sh to startup
		cp -f /var/ossec/modules/clamav/template/stunnel-aix.sh /etc/rc.d/rc2.d/S99stunnel.sh

		# start stunnel
		/opt/freeware/bin/stunnel /opt/freeware/etc/stunnel/stunnel.conf

	fi
}


###########################
# Main
###########################
if [[ $DEBUG -ge 1 ]]; then
	log_event "DEBUG0: malware scanner script started: $0 $ACTION $ARGS"
	log_event "DEBUG0: malware scanner script: DIST: ($DIST) PKG=($PKG) CLAMAV_SERVER=($CLAMAV_SERVER)"
fi

dist_detection

if [ "$1" == "add" ] || [ "$1" == "delete" ]; then
    ACTION=$1
    shift
    shift
    ARGS="$@"
else
    read -r stdin_data
    ARGS=$(echo "$stdin_data" | jq -r .parameters.alert.data.srcip)
fi

get_hub_ip




if [[ "$ARGS" == "update"* ]]; then
	freshclam_update

elif [[ "$ARGS" == "init-local"* ]]; then

	log_event "malware scanner initializing local config"

	if [[ $DIST == "aix"* ]]; then
		init_clamav_conf
		freshclam_update
		/etc/rc.d/init.d/clamav start
	elif [[ $DISTRO == "rh9like" ]]; then
	    install_clamav
		init_clamav_conf
		freshclam_update
		systemctl restart clamd@scan
		RETVAL=$?
		if [ $RETVAL -eq 0 ]; then
			log_event "malware scanner initialization completed successfully"
		else
			log_event "malware scanner initialization failed (001)"
		fi
	else
		install_clamav
		init_clamav_conf
		freshclam_update
		systemctl restart clamav-daemon
		RETVAL=$?
		if [ $RETVAL -eq 0 ]; then
			log_event "malware scanner initialization completed successfully"
		else
			log_event "malware scanner initialization failed (002)"
		fi
		
	fi
# Planned
elif [[ "$ARGS" == "init-remote"* ]]; then
	log_event "malware scanner initializing remote config"
	if [[ $DIST == "aix"* ]]; then
		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG0: init-remote detected AIX"
		fi
		init_clamav_remote_conf
	fi
else 
    log_event "running malware scan on ${ARGS}" 
	TARGET=${ARGS%% scanid:*}
	SCANID=${ARGS##*scanid:}

	if [[ $DIST == "aix"* ]]; then
		CLAM_PATH=/opt/freeware/bin 
		if [ -f /opt/freeware/etc/clamd.conf ]; then
			CLAM_CONF=/opt/freeware/etc/clamd.conf
		elif [ -f /etc/clamd-remote.conf ]; then
			CLAM_CONF=/etc/clamd-remote.conf
		else
			CLAMAV_SERVER=local
		fi
	
	elif [[ $DISTRO == "rh9like" ]]; then
		CLAM_PATH=/usr/bin 
		CLAM_CONF=/etc/clamd.d/scan.conf

	# Solaris
	elif [[ $PKG == "pkg" ]]; then
		CLAM_PATH=/opt/csw/bin/
		CLAM_CONF=/etc/opt/csw/clamd.conf
	else
		CLAM_PATH=/usr/bin
        if [ -f /etc/clamd.conf ]; then
            CLAM_CONF=/etc/clamd.conf
        elif [ -f /etc/clamav/clamd.conf ]; then
            CLAM_CONF=/etc/clamav/clamd.conf
        elif [ -f /etc/clamd.d/scan.conf ]; then
            CLAM_CONF=/etc/clamd.d/scan.conf
        fi
		
	fi


	# Scan Type
	if [[ $CLAMAV_SERVER == "daemon" ]]; then
		# remote scanner
		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG1: $CLAM_PATH/clamdscan -c $CLAM_CONF -i -m --no-summary $OPTIONS ${TARGET}"
		fi
		RESULT=$($CLAM_PATH/clamdscan -c $CLAM_CONF -i -m  --no-summary $OPTIONS ${TARGET})
	else
		# local
		RESULT=$($CLAM_PATH/clamscan -i --no-summary $OPTIONS ${TARGET})
	fi

	IFS=$'\n'
	for line in $RESULT; do
		# if DIST is aix or solaris, use logger
		if [[ $DIST == "aix"* || $DIST == "solaris" ]]; then
			DATE=$(date "+%b %d %H:%M:%S")
			echo "${DATE} `hostname` clamd: $line scanid:${SCANID}" >> $LOG_FILE
		else

      		logger -t clamd "$line scanid:${SCANID}"
		fi

	done
	unset IFS
    log_event "completed malware scan on ${TARGET}" 

fi

